@@ -23,7 +23,7 @@
 #include <goffice/graph/gog-object-xml.h>
 #include <goffice/graph/gog-object.h>
 #include <goffice/graph/gog-plot.h>
-#include <goffice/graph/gog-reg-curve.h>
+#include <goffice/graph/gog-trend-line.h>
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/data/go-data.h>
 #include <goffice/utils/go-color.h>
@@ -116,7 +116,7 @@ gog_object_set_arg_full (char const *name, char const *val, GogObject *obj, xmlN
 				g_object_unref (val_obj);
 			}
 		}
-	} else if (!gsf_xml_gvalue_from_str (&res, G_TYPE_FUNDAMENTAL (prop_type), val))
+	} else if (!gsf_xml_gvalue_from_str (&res, prop_type, val))
 		success = FALSE;
 
 	if (!success) {
@@ -144,8 +144,11 @@ gog_object_write_property_sax (GogObject const *obj, GParamSpec *pspec, GsfXMLOu
 	g_object_get_property  (G_OBJECT (obj), pspec->name, &value);
 
 	/* No need to save default values */
-	if (!(pspec->flags & GOG_PARAM_FORCE_SAVE) &&
-	    g_param_value_defaults (pspec, &value)) {
+	if (((pspec->flags & GOG_PARAM_POSITION) &&
+	     gog_object_is_default_position_flags (obj, pspec->name)) ||
+	    (!(pspec->flags & GOG_PARAM_FORCE_SAVE) &&
+	     !(pspec->flags & GOG_PARAM_POSITION) && 
+	     g_param_value_defaults (pspec, &value))) {
 		g_value_unset (&value);
 		return;
 	}
@@ -159,7 +162,9 @@ gog_object_write_property_sax (GogObject const *obj, GParamSpec *pspec, GsfXMLOu
 	case G_TYPE_LONG:
 	case G_TYPE_ULONG:
 	case G_TYPE_FLOAT:
-	case G_TYPE_DOUBLE: {
+	case G_TYPE_DOUBLE:
+	case G_TYPE_ENUM:
+	case G_TYPE_FLAGS: {
 		GValue str = { 0 };
 		g_value_init (&str, G_TYPE_STRING);
 		g_value_transform (&value, &str);
@@ -196,7 +201,8 @@ gog_object_write_property_sax (GogObject const *obj, GParamSpec *pspec, GsfXMLOu
 		break;
 
 	default:
-		;
+		g_warning ("I could not persist property \"%s\", since type \"%s\" is unhandled.",
+			   g_param_spec_get_name (pspec), g_type_name (G_TYPE_FUNDAMENTAL(prop_type)));
 	}
 	g_value_unset (&value);
 }
@@ -315,10 +321,8 @@ gog_object_new_from_xml (GogObject *parent, xmlNode *node)
 			GogPlot *plot = gog_plot_new_by_name (type_name);
 			if (plot)
 				res = GOG_OBJECT (plot);
-			else {
-				GogRegCurve *curve = gog_reg_curve_new_by_name (type_name);
-				res = GOG_OBJECT (curve);
-			}
+			else
+				res = GOG_OBJECT (gog_trend_line_new_by_name (type_name));
 		} else
 			res = g_object_new (type, NULL);
 		xmlFree (type_name);
@@ -553,7 +557,7 @@ gogo_start (GsfXMLIn *xin, xmlChar const **attrs)
 		if (t == 0) {
 			res = (GogObject *)gog_plot_new_by_name (type);
 			if (NULL == res)
-				res = (GogObject *)gog_reg_curve_new_by_name (type);
+				res = (GogObject *)gog_trend_line_new_by_name (type);
 		} else
 			res = g_object_new (t, NULL);
 
