@@ -174,7 +174,7 @@ set_dash (GogRendererGnomePrint *prend, ArtVpathDash *dash)
 }
 
 static void
-gog_renderer_gnome_print_draw_path (GogRenderer *renderer, ArtVpath const *path)
+gog_renderer_gnome_print_draw_path (GogRenderer *renderer, ArtVpath const *path, gboolean sharp)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -213,7 +213,7 @@ print_image (GogRendererGnomePrint *prend, GdkPixbuf *image, int w, int h)
 #define PIXBUF_SIZE 1024
 static void
 gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *path,
-				       gboolean narrow)
+				       gboolean sharp, gboolean narrow)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -434,14 +434,14 @@ gog_renderer_gnome_print_draw_bezier_polygon (GogRenderer *rend, ArtBpath const
 					      gboolean narrow)
 {
 	ArtVpath *vpath = art_bez_path_to_vec (path, .1);
-	gog_renderer_gnome_print_draw_polygon (rend, vpath, narrow);
+	gog_renderer_gnome_print_draw_polygon (rend, vpath, FALSE, narrow);
 	art_free (vpath);
 }
 
 static void
 gog_renderer_gnome_print_draw_text (GogRenderer *rend, char const *text,
 				    GogViewAllocation const *pos, GtkAnchorType anchor,
-				    GogViewAllocation *result)
+				    gboolean use_markup)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
 	PangoFontDescription *pango_font = get_font (prend,  rend->cur_style->font.font);
@@ -453,7 +453,10 @@ gog_renderer_gnome_print_draw_text (GogRenderer *rend, char const *text,
 		int iw, ih;
 
 		pango_layout_set_font_description (prend->layout, pango_font);
-		pango_layout_set_markup (prend->layout, text, -1);
+		if (use_markup)
+			pango_layout_set_markup (prend->layout, text, -1);
+		else
+			pango_layout_set_text (prend->layout, text, -1);
 		pango_layout_get_size (prend->layout, &iw, &ih);
 		obr.w = iw / (double)PANGO_SCALE;
 		obr.h = ih / (double)PANGO_SCALE;
@@ -497,24 +500,22 @@ gog_renderer_gnome_print_draw_text (GogRenderer *rend, char const *text,
 		gnome_print_rotate (prend->gp_context, rend->cur_style->text_layout.angle);
 		gnome_print_pango_layout (prend->gp_context, prend->layout);
 		gnome_print_grestore (prend->gp_context);
-		if (result != NULL) {
-			result->x = aabr.x;
-			result->y = aabr.y;
-			result->w = aabr.w;
-			result->h = aabr.h;
-		}
 	}
 }
 
 static void
-gog_renderer_gnome_print_get_text_OBR (GogRenderer *rend, char const *text, GOGeometryOBR *obr)
+gog_renderer_gnome_print_get_text_OBR (GogRenderer *rend, char const *text, 
+				       gboolean use_markup, GOGeometryOBR *obr)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
 	PangoFontDescription *pango_font = get_font (prend,  rend->cur_style->font.font);
 	int iw, ih;
 
 	pango_layout_set_font_description (prend->layout, pango_font);
-	pango_layout_set_markup (prend->layout, text, -1);
+	if (use_markup)
+		pango_layout_set_markup (prend->layout, text, -1);
+	else
+		pango_layout_set_text (prend->layout, text, -1);
 	pango_layout_get_size (prend->layout, &iw, &ih);
 	obr->w = iw / (double)PANGO_SCALE;
 	obr->h = ih / (double)PANGO_SCALE;
@@ -631,6 +632,7 @@ gog_graph_print_to_gnome_print (GogGraph *graph,
 	 * http://bugzilla.gnome.org/show_bug.cgi?id=149452
 	 */
 	gnome_print_setlinewidth (prend->gp_context, 0.1);
+	gnome_print_setlinecap (prend->gp_context, 1);
 	
 	gog_view_render	(prend->base.view, NULL);
 	g_object_unref (prend);
