@@ -73,8 +73,9 @@ typedef enum
 G_BEGIN_DECLS
 
 /* We need to define an hair line width for the svg and gnome_print renderer. 
- * 0.24 pt is the dot size of a 300 dpi printer, if the plot is printed at scale 1:1 */
-#define GOG_RENDERER_HAIR_LINE_WIDTH	0.24
+ * 0.5 pt is approx. the dot size of a 150 dpi printer, if the plot is 
+ * printed at scale 1:1 */
+#define GOG_RENDERER_HAIRLINE_WIDTH_PTS	0.5
 
 #define GOG_RENDERER_GRIP_SIZE	4
 
@@ -116,7 +117,7 @@ void  gog_renderer_draw_bezier_path     (GogRenderer *rend, ArtBpath const *path
 
 void  gog_renderer_draw_text	  (GogRenderer *rend, char const *text,
 				   GogViewAllocation const *pos, GtkAnchorType anchor,
-				   GogViewAllocation *result);
+				   gboolean use_markup);
 void  gog_renderer_draw_marker	  (GogRenderer *rend, double x, double y);
 void  gog_renderer_draw_grip	  (GogRenderer *renderer, double x, double y);
 
@@ -125,8 +126,10 @@ void  gog_renderer_draw_grip	  (GogRenderer *renderer, double x, double y);
 						 (y) >= ((grip_y) - (GOG_RENDERER_GRIP_SIZE)) && \
 						 (y) <= ((grip_y) + (GOG_RENDERER_GRIP_SIZE)))
 
-void  gog_renderer_get_text_OBR   (GogRenderer *rend, char const *text, GOGeometryOBR *obr);
-void  gog_renderer_get_text_AABR  (GogRenderer *rend, char const *text, GOGeometryAABR *aabr);
+void  gog_renderer_get_text_OBR   (GogRenderer *rend, char const *text, 
+				   gboolean use_markup, GOGeometryOBR *obr);
+void  gog_renderer_get_text_AABR  (GogRenderer *rend, char const *text, 
+				   gboolean use_markup, GOGeometryAABR *aabr);
 
 #define gog_renderer_draw_arc(r,cx,cy,rx,ry,th0,th1) \
 	gog_renderer_draw_ring_wedge (r,cx,cy,rx,ry,-1.,-1.,th0,th1,FALSE)
@@ -139,6 +142,8 @@ double gog_renderer_pt2r_x   	  	(GogRenderer const *r, double d);
 double gog_renderer_pt2r_y   	  	(GogRenderer const *r, double d);
 double gog_renderer_pt2r   	  	(GogRenderer const *r, double d);
 
+double gog_renderer_get_hairline_width_pts	(GogRenderer const *r);
+
 /* utilities for cairo/libart transition */
 GogRenderer 	*gog_renderer_new_for_pixbuf 	(GogGraph *graph);
 gboolean	 gog_renderer_update		(GogRenderer *renderer, double w, double h, double zoom);
