@@ -2,6 +2,8 @@
 /**
  * go-format-sel.c: A widget to select a format
  *
+ * Copyright (C) 2006-2007 Morten Welinder (terra@gnome.org)
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -20,27 +22,19 @@
 #include <goffice/goffice-config.h>
 #include "go-format-sel.h"
 
-#include <goffice/gtk/goffice-gtk.h>
-#include <goffice/gtk/go-combo-text.h>
-#include <goffice/utils/format-impl.h>
-#include <goffice/utils/go-color.h>
-#include <goffice/utils/go-marshalers.h>
+#include "goffice/gtk/goffice-gtk.h"
+#include "goffice/gtk/go-combo-text.h"
+#include "goffice/utils/go-format.h"
+#include "goffice/utils/go-locale.h"
+#include "goffice/utils/go-color.h"
+#include "goffice/utils/go-marshalers.h"
+#include "goffice/utils/go-glib-extras.h"
 
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtksizegroup.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtktextview.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtktreeview.h>
+#include <gtk/gtk.h>
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
 
 #include <string.h>
-#include <locale.h>
 
 /* The maximum number of chars in the formatting sample */
 #define FORMAT_PREVIEW_MAX 25
@@ -106,10 +100,10 @@ typedef enum {
 	F_ENTRY,
 	F_LIST_LABEL,		F_LIST_SCROLL,		F_LIST,
 	F_DECIMAL_SPIN,		F_ENGINEERING_BUTTON,	
-	F_SUPERSCRIPT_BOX,	F_SUPERSCRIPT_BUTTON,	F_SUPERSCRIPT_HIDE_1_BUTTON,
+	F_SUPERSCRIPT_BUTTON,	F_SUPERSCRIPT_HIDE_1_BUTTON,
 	F_NEGATIVE_LABEL,	F_NEGATIVE_SCROLL,	F_NEGATIVE,
-	F_DECIMAL_LABEL,	F_CODE_LABEL,	F_SYMBOL_BOX,
-	F_DECIMAL_BOX,	F_CODE_BOX,	F_MAX_WIDGET
+	F_DECIMAL_LABEL,	F_CODE_LABEL,
+	F_MAX_WIDGET
 } FormatWidget;
 
 struct  _GOFormatSel {
@@ -178,32 +172,181 @@ static guint go_format_sel_signals [LAST_SIGNAL] = { 0 };
 static void format_entry_set_text (GOFormatSel *gfs, const gchar *text);
 
 static void
-generate_format (GOFormatSel *gfs)
+generate_number (GString *dst,
+		 int num_decimals,
+		 gboolean thousands_sep,
+		 int symbol,
+		 int negative_fmt)
 {
-	GOFormatFamily const page = gfs->format.current_type;
-	GOFormat *new_format;
+	gboolean precedes = go_format_currencies[symbol].precedes;
+	gboolean has_space = go_format_currencies[symbol].has_space;
+	const char *symstr = go_format_currencies[symbol].symbol;
 
-	/* 
-	 * It is a strange idea not to reuse GOFormatDetails
-	 * in this file, so build one.
-	 */
-	GOFormatDetails format = gfs->format.spec->family_info;
-	format.thousands_sep = gfs->format.use_separator;
-	format.num_decimals = gfs->format.num_decimals;
-	format.negative_fmt = gfs->format.negative_format;
-	format.currency_symbol_index = gfs->format.currency_index;
-	format.use_markup = gfs->format.use_markup;
-	format.exponent_step = gfs->format.exponent_step;
-	format.simplify_mantissa = gfs->format.simplify_mantissa;
-
-	new_format = go_format_new (page, &format);
-	if (new_format) {
-		char *tmp = go_format_as_XL (new_format, TRUE);
-		format_entry_set_text (gfs, tmp);
-		g_free (tmp);
+	/* Currency */
+	if (symbol != 0 && precedes) {
+		g_string_append (dst, symstr);
+		if (has_space) g_string_append_c (dst, ' ');
 	}
 
-	go_format_unref (new_format);
+	if (thousands_sep)
+		g_string_append (dst, "#,##0");
+	else
+		g_string_append_c (dst, '0');
+
+	if (num_decimals > 0) {
+		g_string_append_c (dst, '.');
+		go_string_append_c_n (dst, '0', num_decimals);
+	}
+
+	/* Currency */
+	if (symbol != 0 && !precedes) {
+		if (has_space)
+			g_string_append_c (dst, ' ');
+		g_string_append (dst, symstr);
+	}
+
+	/* There are negatives */
+	if (negative_fmt > 0) {
+		size_t prelen = dst->len;
+
+		switch (negative_fmt) {
+		case 1 : g_string_append (dst, ";[Red]");
+			break;
+		case 2 : g_string_append (dst, "_);(");
+			break;
+		case 3 : g_string_append (dst, "_);[Red](");
+			break;
+		default :
+			g_assert_not_reached ();
+		};
+
+		g_string_append_len (dst, dst->str, prelen);
+
+		if (negative_fmt >= 2)
+			g_string_append_c (dst, ')');
+	}
+}
+
+static void
+generate_accounting (GString *dst,
+		     int num_decimals,
+		     int symbol)
+{
+	GString *num = g_string_new (NULL);
+	GString *sym = g_string_new (NULL);
+	GString *q = g_string_new (NULL);
+
+	gboolean precedes = go_format_currencies[symbol].precedes;
+	gboolean has_space = go_format_currencies[symbol].has_space;
+	const char *symstr = go_format_currencies[symbol].symbol;
+	const char *quote = symstr[0] != '[' ? "\"" : "";
+
+	generate_number (num, num_decimals, TRUE, 0, 0);
+	go_string_append_c_n (q, '?', num_decimals);
+
+	if (precedes) {
+		g_string_append (sym, quote);
+		g_string_append (sym, symstr);
+		g_string_append (sym, quote);
+		g_string_append (sym, "* ");
+		if (has_space) g_string_append_c (sym, ' ');
+
+		g_string_append_printf
+			(dst,
+			 "_(%s%s_);_(%s(%s);_(%s\"-\"%s_);_(@_)",
+			 sym->str, num->str,
+			 sym->str, num->str,
+			 sym->str, q->str);
+	} else {
+		g_string_append (sym, "* ");
+		if (has_space) g_string_append_c (sym, ' ');
+		g_string_append (sym, quote);
+		g_string_append (sym, symstr);
+		g_string_append (sym, quote);
+
+		g_string_append_printf
+			(dst,
+			 "_(%s%s_);_((%s)%s;_(\"-\"%s%s_);_(@_)",
+			 num->str, sym->str,
+			 num->str, sym->str,
+			 q->str, sym->str);
+	}
+
+	g_string_free (num, TRUE);
+	g_string_free (q, TRUE);
+	g_string_free (sym, TRUE);
+}
+
+static void
+generate_scientific (GString *dst,
+		     int num_decimals,
+		     int exponent_step,
+		     gboolean use_markup,
+		     gboolean simplify_mantissa)
+{
+	go_string_append_c_n (dst, '#', MAX (0, exponent_step - 1));
+	if (simplify_mantissa)
+		g_string_append_c (dst, '#');
+	else
+		g_string_append_c (dst, '0');
+
+	if (num_decimals > 0) {
+		g_string_append_c (dst, '.');
+		go_string_append_c_n (dst, '0', num_decimals);
+	}
+
+	if (use_markup)
+		g_string_append (dst, "EE0");
+	else
+		g_string_append (dst, "E+00");
+}
+
+
+static char *
+generate_format (GOFormatSel *gfs, GOFormatFamily page)
+{
+	GString *fmt = g_string_new (NULL);
+
+	switch (page) {
+	case GO_FORMAT_GENERAL:
+	case GO_FORMAT_TEXT:
+		g_string_append (fmt, go_format_builtins[page][0]);
+		break;
+	case GO_FORMAT_NUMBER:
+		generate_number (fmt,
+				 gfs->format.num_decimals,
+				 gfs->format.use_separator,
+				 0,
+				 gfs->format.negative_format);
+		break;
+	case GO_FORMAT_CURRENCY:
+		generate_number (fmt,
+				 gfs->format.num_decimals,
+				 gfs->format.use_separator,
+				 gfs->format.currency_index,
+				 gfs->format.negative_format);
+		break;
+	case GO_FORMAT_ACCOUNTING:
+		generate_accounting (fmt,
+				     gfs->format.num_decimals,
+				     gfs->format.currency_index);
+		break;
+	case GO_FORMAT_PERCENTAGE:
+		generate_number (fmt, gfs->format.num_decimals, FALSE, 0, 0);
+		g_string_append_c (fmt, '%');
+		break;
+	case GO_FORMAT_SCIENTIFIC:
+		generate_scientific (fmt,
+				     gfs->format.num_decimals,
+				     gfs->format.exponent_step,
+				     gfs->format.use_markup,
+				     gfs->format.simplify_mantissa);
+		break;
+	default:
+		break;
+	}
+
+	return g_string_free (fmt, fmt->len == 0);
 }
 
 static char *
@@ -219,13 +362,22 @@ generate_preview (GOFormatSel *gfs, GOColor *c)
 static void
 draw_format_preview (GOFormatSel *gfs, gboolean regen_format)
 {
-	char		*preview;
-	GOFormat	*sf = NULL;
-	GOColor		 c = 0;
-	GdkColor	 gdk_color;
+	char *preview;
+	GOFormat *sf = NULL;
+	GOColor c = 0;
+	GdkColor gdk_color;
+	gsize len;
 
-	if (regen_format)
-		generate_format (gfs);
+	if (regen_format) {
+		char *fmt = generate_format (gfs, gfs->format.current_type);
+
+		if (fmt) {
+			char *lfmt = go_format_str_localize (fmt);
+			format_entry_set_text (gfs, lfmt);
+			g_free (lfmt);
+			g_free (fmt);
+		}
+	}
 
 	if (NULL == (sf = gfs->format.spec))
 		return;
@@ -233,13 +385,17 @@ draw_format_preview (GOFormatSel *gfs, gboolean regen_format)
 	if (NULL == (preview = generate_preview (gfs, &c)))
 		return;
 
-	if (strlen (preview) > FORMAT_PREVIEW_MAX)
-		strcpy (&preview[FORMAT_PREVIEW_MAX - 5], " ...");
+	len = g_utf8_strlen (preview, -1);
+	if (len > FORMAT_PREVIEW_MAX)
+		strcpy (g_utf8_offset_to_pointer (preview, FORMAT_PREVIEW_MAX - 5),
+			"...");
+
 	gtk_text_buffer_set_text (gfs->format.preview_buffer, preview, -1);
 	if (c != 0)
 		go_color_to_gdk (c, &gdk_color);
 	else
 		gdk_color_parse ("black", &gdk_color);
+
 	gtk_widget_modify_text (GTK_WIDGET (gfs->format.preview), 
 				GTK_STATE_NORMAL, &gdk_color);
 	g_free (preview);
@@ -248,20 +404,7 @@ draw_format_preview (GOFormatSel *gfs, gboolean regen_format)
 static void
 fillin_negative_samples (GOFormatSel *gfs)
 {
-	static char const *const decimals = "098765432109876543210987654321";
-	static char const *const formats[4] = {
-		"-%s%s3%s210%s%s%s%s",
-		"%s%s3%s210%s%s%s%s",
-		"(%s%s3%s210%s%s%s%s)",
-		"(%s%s3%s210%s%s%s%s)"
-	};
-	int const n = 30 - gfs->format.num_decimals;
-
 	GOFormatFamily const page = gfs->format.current_type;
-	char const *space_b = "", *currency_b;
-	char const *space_a = "", *currency_a;
-	const char *decimal;
-	const char *thousand_sep;
 	int i;
 	GtkTreeIter  iter;
 	GtkTreePath *path;
@@ -269,52 +412,25 @@ fillin_negative_samples (GOFormatSel *gfs)
 	SETUP_LOCALE_SWITCH;
 
 	g_return_if_fail (page == GO_FORMAT_NUMBER || page == GO_FORMAT_CURRENCY);
-	g_return_if_fail (gfs->format.num_decimals <= 30);
 
 	START_LOCALE_SWITCH;
 
-	if (gfs->format.use_separator)
-		thousand_sep = format_get_thousand ()->str;
-	else
-		thousand_sep = "";
-	if (gfs->format.num_decimals > 0)
-		decimal = format_get_decimal ()->str;
-	else
-		decimal = "";
-
-	if (page == GO_FORMAT_CURRENCY) {
-		currency_b = go_format_currencies[gfs->format.currency_index].symbol;
-		/*
-		 * FIXME : This should be better hidden.
-		 * Ideally the render would do this for us.
-		 */
-		if (currency_b[0] == '[' && currency_b[1] == '$') {
-			char const *end = strchr (currency_b+2, '-');
-			if (end == NULL)
-				end = strchr (currency_b+2, ']');
-			currency_b = g_strndup (currency_b+2, end-currency_b-2);
-		} else
-			currency_b = g_strdup (currency_b);
-
-		if (go_format_currencies[gfs->format.currency_index].has_space)
-			space_b = " ";
-
-		if (!go_format_currencies[gfs->format.currency_index].precedes) {
-			currency_a = currency_b;
-			currency_b = "";
-			space_a = space_b;
-			space_b = "";
-		} else {
-			currency_a = "";
-		}
-	} else
-		currency_a = currency_b = "";
-
 	more = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (gfs->format.negative_types.model), &iter);
 	for (i = 0 ; i < 4; i++) {
-		char *buf = g_strdup_printf (formats[i],
-					     currency_b, space_b, thousand_sep, decimal,
-					     decimals + n, space_a, currency_a);
+		GString *fmtstr = g_string_new (NULL);
+		GOFormat *fmt;
+		char *buf;
+
+		generate_number (fmtstr,
+				 gfs->format.num_decimals,
+				 gfs->format.use_separator,
+				 page == GO_FORMAT_NUMBER ? 0 : gfs->format.currency_index,
+				 i);
+		fmt = go_format_new_from_XL (fmtstr->str);
+		g_string_free (fmtstr, TRUE);
+		buf = go_format_value (fmt, -3210.123456789);
+		go_format_unref (fmt);
+
 		if (!more)
 			gtk_list_store_append (gfs->format.negative_types.model, &iter);
 		gtk_list_store_set (gfs->format.negative_types.model, &iter,
@@ -329,12 +445,6 @@ fillin_negative_samples (GOFormatSel *gfs)
 		g_free (buf);
 	}
 
-	/* If non empty then free the string */
-	if (*currency_a)
-		g_free ((char*)currency_a);
-	if (*currency_b)
-		g_free ((char*)currency_b);
-
 	path = gtk_tree_path_new ();
 	gtk_tree_path_append_index (path, gfs->format.negative_format);
 	gtk_tree_selection_select_path (gfs->format.negative_types.selection, path);
@@ -403,11 +513,11 @@ fmt_dialog_init_fmt_list (GOFormatSel *gfs, char const *const *formats,
 {
 	GtkTreeIter iter;
 	char *fmt;
-	char const *cur_fmt = gfs->format.spec->format;
+	const char *cur_fmt = go_format_as_XL (gfs->format.spec);
 
 	for (; *formats; formats++) {
 		gtk_list_store_append (gfs->format.formats.model, &iter);
-		fmt = go_format_str_as_XL (*formats, TRUE);
+		fmt = go_format_str_localize (*formats);
 		gtk_list_store_set (gfs->format.formats.model, &iter,
 				    0, fmt, -1);
 		g_free (fmt);
@@ -417,6 +527,25 @@ fmt_dialog_init_fmt_list (GOFormatSel *gfs, char const *const *formats,
 	}
 }
 
+static const char *
+find_builtin (const char *fmtstr, int page, gboolean def)
+{
+	int list_elem = 0;
+	char const * const *candidates = go_format_builtins[page];
+
+	while (candidates[list_elem]) {
+		if (strcmp (candidates[list_elem], fmtstr) == 0)
+			break;
+		list_elem++;
+	}
+
+	if (candidates[list_elem] == NULL && def)
+		list_elem = 0;
+
+	return candidates[list_elem];
+}
+
+
 static void
 fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 {
@@ -430,7 +559,6 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 		/* Number */
 		{
 			F_NUMBER_EXPLANATION,
-			F_DECIMAL_BOX,
 			F_DECIMAL_LABEL,
 			F_DECIMAL_SPIN,
 			F_SEPARATOR,
@@ -442,11 +570,9 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 		/* Currency */
 		{
 			F_CURRENCY_EXPLANATION,
-			F_DECIMAL_BOX,
 			F_DECIMAL_LABEL,
 			F_DECIMAL_SPIN,
 			F_SEPARATOR,
-			F_SYMBOL_BOX,
 			F_SYMBOL_LABEL,
 			F_SYMBOL,
 			F_NEGATIVE_LABEL,
@@ -457,10 +583,8 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 		/* Accounting */
 		{
 			F_ACCOUNTING_EXPLANATION,
-			F_DECIMAL_BOX,
 			F_DECIMAL_LABEL,
 			F_DECIMAL_SPIN,
-			F_SYMBOL_BOX,
 			F_SYMBOL_LABEL,
 			F_SYMBOL,
 			F_MAX_WIDGET
@@ -484,7 +608,6 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 		/* Percentage */
 		{
 			F_PERCENTAGE_EXPLANATION,
-			F_DECIMAL_BOX,
 			F_DECIMAL_LABEL,
 			F_DECIMAL_SPIN,
 			F_MAX_WIDGET
@@ -500,11 +623,9 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 		/* Scientific */
 		{
 			F_SCIENTIFIC_EXPLANATION,
-			F_DECIMAL_BOX,
 			F_DECIMAL_LABEL,
 			F_DECIMAL_SPIN,
 			F_ENGINEERING_BUTTON,
-			F_SUPERSCRIPT_BOX,
 			F_SUPERSCRIPT_BUTTON,
 			F_SUPERSCRIPT_HIDE_1_BUTTON,
 			F_MAX_WIDGET
@@ -522,7 +643,6 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 		/* Custom */
 		{
 			F_CUSTOM_EXPLANATION,
-			F_CODE_BOX,
 			F_CODE_LABEL,
 			F_ENTRY,
 			F_LIST_LABEL,
@@ -557,14 +677,13 @@ stays:
 	    page == GO_FORMAT_ACCOUNTING ||
 	    page == GO_FORMAT_FRACTION ||
 	    page == GO_FORMAT_TEXT) {
-		int list_elem = 0;
-		char *tmp;
-		if (page == gfs->format.spec->family)
-			list_elem = gfs->format.spec->family_info.list_element;
+		const char *fmtstr = go_format_as_XL (gfs->format.spec);
+		const char *elem = find_builtin (fmtstr, page, TRUE);
+		char *lelem;
 
-		tmp = go_format_str_as_XL (go_format_builtins[page][list_elem], TRUE);
-		format_entry_set_text (gfs, tmp);
-		g_free (tmp);
+		lelem = go_format_str_localize (elem);
+		format_entry_set_text (gfs, lelem);
+		g_free (lelem);
 	}
 
 	gfs->format.current_type = page;
@@ -604,7 +723,7 @@ stays:
 			*      the std formats and the custom formats in the GOFormat hash.
 			*/
 			if  (page == FMT_CUSTOM && select.stamp == 0) {
-				char *tmp = go_format_as_XL (gfs->format.spec, TRUE);
+				char *tmp = go_format_str_localize (go_format_as_XL (gfs->format.spec));
 				format_entry_set_text (gfs, tmp);
 				g_free (tmp);
 			} else if (select.stamp == 0)
@@ -635,11 +754,6 @@ stays:
 						   gfs->format.num_decimals);
 			break;
 
-		case F_SUPERSCRIPT_BOX:
-			if (!gfs->show_format_with_markup)
-				show_widget = FALSE;
-			break;
-
 		case F_SUPERSCRIPT_BUTTON:
 			if (gfs->show_format_with_markup) {
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
@@ -673,8 +787,12 @@ stays:
 			; /* Nothing */
 		}
 
-		if (show_widget)
-			gtk_widget_show (w);
+		if (show_widget) {
+			while (!GTK_WIDGET_VISIBLE (w)) {
+				gtk_widget_show (w);
+				w = w->parent;
+			}
+		}
 	}
 
 	draw_format_preview (gfs, TRUE);
@@ -715,18 +833,22 @@ static void
 cb_format_entry_changed (GtkEditable *w, GOFormatSel *gfs)
 {
 	char *fmt;
+	const char *cur_fmt;
 	if (!gfs->enable_edit)
 		return;
 
 	fmt = go_format_str_delocalize (gtk_entry_get_text (GTK_ENTRY (w)));
-	if (strcmp (gfs->format.spec->format, fmt)) {
+	cur_fmt = go_format_as_XL (gfs->format.spec);
+
+	if (strcmp (cur_fmt, fmt)) {
 		go_format_unref (gfs->format.spec);
-		gfs->format.spec = go_format_new_from_XL (fmt, FALSE);
+		gfs->format.spec = go_format_new_from_XL (fmt);
 		g_signal_emit (G_OBJECT (gfs),
 			       go_format_sel_signals [FORMAT_CHANGED], 0,
 			       fmt);
 		draw_format_preview (gfs, FALSE);
 	}
+
 	g_free (fmt);
 }
 
@@ -779,7 +901,8 @@ cb_format_currency_select (G_GNUC_UNUSED GtkWidget *ct,
 			break;
 		}
 
-	if (gfs->format.current_type == 1 || gfs->format.current_type == 2)
+	if (gfs->format.current_type == GO_FORMAT_NUMBER ||
+	    gfs->format.current_type == GO_FORMAT_CURRENCY)
 		fillin_negative_samples (gfs);
 	draw_format_preview (gfs, TRUE);
 
@@ -840,6 +963,169 @@ set_format_category (GOFormatSel *gfs, int row)
 	gtk_tree_path_free (path);
 }
 
+/* Returns the index in go_format_currencies of the symbol in ptr */
+static int
+find_symbol (char const *ptr, gsize len, gboolean precedes)
+{
+	int i;
+	gboolean has_space;
+	gboolean quoted;
+
+	if (len <= 0)
+		return 0;
+
+	if (precedes) {
+		has_space = ptr[len - 1] == ' ';
+		if (has_space)
+			len--;
+	} else {
+		has_space = ptr[0] == ' ';
+		if (has_space)
+			len--, ptr++;
+	}
+
+	quoted = len > 2 && ptr[0] == '\"' && ptr[len - 1] == '\"';
+
+	for (i = 1; go_format_currencies[i].symbol; i++) {
+		const GOFormatCurrency *ci = go_format_currencies + i;
+
+		if (ci->precedes != precedes)
+			continue;
+
+		if (strncmp (ci->symbol, ptr, len) == 0) {
+			return i;
+		}
+
+		/* Allow quoting of things that aren't [$FOO] */
+		if (quoted && ci->symbol[0] != '[' &&
+		    strncmp (ci->symbol, ptr + 1, len - 2) == 0) {
+			return i;
+		}
+	}
+
+	return 0;
+}
+
+static GOFormatFamily
+study_format (GOFormatSel *gfs)
+{
+	const GOFormat *fmt = gfs->format.spec;
+	const char *str = go_format_as_XL (fmt);
+	char *newstr;
+	GOFormatFamily typ = go_format_get_family (fmt);
+
+	gfs->format.num_decimals = 0;	
+	gfs->format.negative_format = 0;
+	gfs->format.currency_index = 0;
+	gfs->format.use_separator = FALSE;
+	gfs->format.use_markup = FALSE;
+	gfs->format.exponent_step = 1;
+	gfs->format.simplify_mantissa = FALSE;
+
+	switch (typ) {
+	case GO_FORMAT_NUMBER:
+	case GO_FORMAT_CURRENCY:
+	case GO_FORMAT_ACCOUNTING:
+	case GO_FORMAT_PERCENTAGE:
+	case GO_FORMAT_SCIENTIFIC: {
+		/*
+		 * These guesses only have to be good enough to work on
+		 * the formats we generate ourselves.
+		 */
+
+		const char *dot = strchr (str, '.');
+
+		if (dot) {
+			while (dot[gfs->format.num_decimals + 1] == '0')
+				gfs->format.num_decimals++;
+		}
+
+		gfs->format.negative_format =
+			(strstr (str, ";[Red]") ? 1 : 0) +
+			(strstr (str, "_);") ? 2 : 0);
+
+		if (str[0] == '_' && str[1] == '(') {
+			const char *start = str + 2;
+			gboolean precedes = start[0] != '#';
+			gsize len = 0;
+
+			if (precedes) {
+				while (start[len] && start[len] != '*')
+					len++;
+			} else {
+				while (start[0] == '0' || start[0] == '.' ||
+				       start[0] == '#' || start[0] == ',')
+					start++;
+				if (start[0] == '*' && start[1])
+					start += 2;
+				while (start[len] && start[len] != '_')
+					len++;
+			}
+
+			gfs->format.currency_index =
+				find_symbol (start, len, precedes);
+			typ = GO_FORMAT_ACCOUNTING;
+		} else {
+			gboolean precedes = str[0] != '0' && str[0] != '#';
+			const char *start;
+			gsize len = 0;
+			int symbol;
+
+			if (precedes) {
+				start = str;
+				while (start[len] && start[len] != '0' && start[len] != '#')
+					len++;
+			} else {
+				start = str + strlen (str);
+				if (start > str && start[-1] == ')')
+					start--;
+				while (start > str && start[-1] != '0' && start[-1] != '#')
+					start--, len++;
+			}
+			symbol = find_symbol (start, len, precedes);
+			if (symbol != 0) {
+				gfs->format.currency_index = symbol;
+				typ = GO_FORMAT_CURRENCY;
+			}
+		}
+
+		gfs->format.use_separator = (strstr (str, "#,##0") != NULL);
+
+		if (typ == GO_FORMAT_SCIENTIFIC) {
+			const char *mend = dot ? dot : strchr (str, 'E');
+			gfs->format.use_markup = (strstr (str, "EE0") != NULL);
+			gfs->format.exponent_step = mend - str;
+			gfs->format.simplify_mantissa = mend != str && mend[-1] == '#';
+		}			
+
+		newstr = generate_format (gfs, typ);
+
+		break;
+	}
+
+	default:
+		newstr = NULL;
+		break;
+	}
+
+	if (newstr) {
+		/* Test the generated format. */
+		if (strcmp (str, newstr)) {
+#if 0
+			g_print ("[%s] <-> [%s]\n", str, newstr);
+#endif
+			typ = FMT_CUSTOM;
+		}
+		g_free (newstr);
+	} else {
+		const char *elem = find_builtin (str, typ, FALSE);
+		if (!elem)
+			typ = FMT_CUSTOM;
+	}
+
+	return typ;
+}
+
 
 static void
 set_format_category_menu_from_style (GOFormatSel *gfs)
@@ -849,7 +1135,8 @@ set_format_category_menu_from_style (GOFormatSel *gfs)
 	g_return_if_fail (IS_GO_FORMAT_SEL (gfs));
 
 	/* Attempt to extract general parameters from the current format */
-	if ((page = gfs->format.spec->family) < 0)
+	page = study_format (gfs);
+	if (page < 0)
 		page = FMT_CUSTOM; /* Default to custom */
 
 	set_format_category (gfs, page);
@@ -891,8 +1178,6 @@ populate_menu (GOFormatSel *gfs)
 			  "changed",
 			  G_CALLBACK (cb_format_class_changed), gfs);
 }
-
-
 /*
  * static void
  * fmt_dialog_init_format_page (FormatState *state)
@@ -925,7 +1210,6 @@ nfs_init (GOFormatSel *gfs)
 		"format_list",
 		"format_number_decimals",
 		"format_engineering_button",
-		"format_superscript_box",
 		"format_superscript_button",
 		"format_superscript_hide_1_button",
 		"format_negatives_label",
@@ -933,9 +1217,6 @@ nfs_init (GOFormatSel *gfs)
 		"format_negatives",
 		"format_decimal_label",
 		"format_code_label",
-		"format_symbol_box",
-		"format_decimal_box",
-		"format_code_box",
 		NULL
 	};
 
@@ -972,15 +1253,7 @@ nfs_init (GOFormatSel *gfs)
 	 * all widgets are already hidden. */
 	gfs->format.current_type = -1;
 
-	/* Even if the format was not recognized it has set intelligent defaults */
-	gfs->format.use_separator = gfs->format.spec->family_info.thousands_sep;
-	gfs->format.num_decimals = gfs->format.spec->family_info.num_decimals;
-	gfs->format.negative_format = gfs->format.spec->family_info.negative_fmt;
-	gfs->format.currency_index = gfs->format.spec->family_info.currency_symbol_index;
-	
-	gfs->format.use_markup = FALSE;
-	gfs->format.simplify_mantissa = FALSE;
-	gfs->format.exponent_step = 1;
+	study_format (gfs);
 
 	gfs->format.preview_box = glade_xml_get_widget (gfs->gui, "preview_box");
 	gfs->format.preview = GTK_TEXT_VIEW (glade_xml_get_widget (gfs->gui, "preview"));
@@ -1110,7 +1383,8 @@ nfs_init (GOFormatSel *gfs)
 	/* Connect signal for format menu */
 	set_format_category_menu_from_style (gfs);
 
-	if ((page = gfs->format.spec->family) < 0)
+	page = go_format_get_family (gfs->format.spec);
+	if (page < 0)
 		page = FMT_CUSTOM; /* Default to custom */
 	fmt_dialog_enable_widgets (gfs, page);
 
@@ -1248,13 +1522,7 @@ go_format_sel_set_style_format (GOFormatSel *gfs,
 
 	gfs->format.spec = style_format;
 
-	gfs->format.use_separator 	= style_format->family_info.thousands_sep;
-	gfs->format.num_decimals 	= style_format->family_info.num_decimals;
-	gfs->format.negative_format 	= style_format->family_info.negative_fmt;
-	gfs->format.currency_index 	= style_format->family_info.currency_symbol_index;
-	gfs->format.use_markup 		= style_format->family_info.use_markup;
-	gfs->format.exponent_step	= style_format->family_info.exponent_step;
-	gfs->format.simplify_mantissa	= style_format->family_info.simplify_mantissa;
+	study_format (gfs);
 
 	combo = GO_COMBO_TEXT (gfs->format.widget[F_SYMBOL]);
 	go_combo_text_set_text
@@ -1333,7 +1601,7 @@ go_format_sel_set_locale (GOFormatSel *gfs,
 char const *    
 go_format_sel_format_classification (GOFormat const *style_format)
 {
-	GOFormatFamily page = style_format->family;
+	GOFormatFamily page = go_format_get_family (style_format);
 
 	if (page < 0 || page > FMT_CUSTOM)
 		page = FMT_CUSTOM; /* Default to custom */
